﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace MVCFY2
{
    #region IPageList
    public interface IPageList
    {
        int TotalPage //总页数
        {
            get;
        }
        int TotalCount//总数量
        {
            get;
            set;
        }

        int PageIndex//第几页
        {
            get;
            set;
        }

        int PageSize//一页显示几条数据
        {
            get;
            set;
        }

        bool IsPreviousPage//是否上一页
        {
            get;
        }

        bool IsNextPage
        {
            get;
        }
    }
    #endregion

    public class PagedList<T> : List<T>, IPageList
    {
        #region 参数初始化
        public int TotalPage
        {
            get
            {
                return (int)Math.Ceiling((double)TotalCount / PageSize);
            }
        }

        public int TotalCount
        {
            get;
            set;
        }

        public int PageIndex
        {
            get;
            set;
        }

        public int PageSize
        {
            get;
            set;
        }

        public bool IsPreviousPage
        {
            get { return (PageIndex > 1); }
        }

        public bool IsNextPage
        {
            get
            {
                return ((PageIndex) * PageSize) < TotalCount;
            }
        }
        #endregion

        #region 构造函数，传入的数据源需要实现IQueryable接口
        /// <summary>
        /// 构造函数，传入的数据源需要实现IQueryable接口
        /// </summary>
        /// <param name="source">数据源</param>
        /// <param name="index">第几页数据</param>
        /// <param name="pageSize"></param>
        public PagedList(IQueryable<T> source, int? pageIndex, int? pageSize)
        {
            if (pageIndex == null) { pageIndex = 1; }//默认显示第一页数据
            if (pageSize == null) { pageSize = 10; }//默认每页显示10条数据
            this.TotalCount = source.Count();
            this.PageSize = pageSize.Value;
            this.PageIndex = pageIndex.Value;

            if (IsPreviousPage == false)
            {
                pageIndex = 1;
            }

            if (IsNextPage == false)
            {
                pageIndex = TotalPage;
            }
            int BeforePageIndex = pageIndex.Value - 1;
            if (BeforePageIndex < 0)
            {
                BeforePageIndex = 0;
            }
            AddRange(source.Skip((BeforePageIndex) * pageSize.Value).Take(pageSize.Value));
            //如果Index是0会报错，不能够是负数
        }
        #endregion

        #region 构造函数，传入的数据源需要实现IEnumerable接口
        public PagedList(IEnumerable<T> source, int? index, int? pageSize)
        {
            if (index == null) { index = 1; }
            if (pageSize == null) { pageSize = 20; }
            this.TotalCount = source.Count();
            this.PageSize = pageSize.Value;
            this.PageIndex = index.Value;
            if (IsPreviousPage == false)
            {
                index = 1;
            }

            if (IsNextPage == false)
            {
                index = TotalPage;
            }
            int BeforePageIndex = index.Value - 1;
            if (BeforePageIndex < 0)
            {
                BeforePageIndex = 0;
            }
            this.AddRange(source.Skip((index.Value - 1) * pageSize.Value).Take(pageSize.Value));
        }
        #endregion
    }

    public static class Pagination
    {
        /// <summary>
        /// 返回第几页的数据
        /// </summary>
        /// <typeparam name="T">返回的数据类型</typeparam>
        /// <param name="source">数据源，数据源需继承IQueryable接口</param>
        /// <param name="index">第几页</param>
        /// <param name="pageSize">每页显示的数据条数，默认每页显示10条数据</param>
        /// <returns></returns>
        public static PagedList<T> ToPagedList<T>(this IQueryable<T> source, int? index, int? pageSize)
        {
            return new PagedList<T>(source, index, pageSize);
        }

        public static PagedList<T> ToPagedList<T>(this IQueryable<T> source, int? index)
        {
            return new PagedList<T>(source, index, 10);
        }
        /// <summary>
        /// 返回第几页的数据
        /// </summary>
        /// <typeparam name="T">返回的数据类型</typeparam>
        /// <param name="source">数据源，数据源需继承IEnumerable接口</param>
        /// <param name="index">第几页</param>
        /// <param name="pageSize">每页显示的数据条数，默认每页显示10条数据</param>
        /// <returns></returns>
        public static PagedList<T> ToPagedList<T>(this IEnumerable<T> source, int? index, int? pageSize)
        {
            return new PagedList<T>(source, index, pageSize);
        }

        public static PagedList<T> ToPagedList<T>(this IEnumerable<T> source, int? index)
        {
            return new PagedList<T>(source, index, 20);
        }
    }
}